import 'dart:convert';
import 'dart:developer';

import 'package:localstore/localstore.dart';

import 'apiResponse.dart';

abstract class RequestCache<T extends ApiResponse?> {
  static const int cacheNothing = 0;
  static const int cacheMinute = 60;
  static const int cacheHour = 60 * 60;
  static const int cacheDay = 60 * 60 * 24;

  int maxCacheTime;
  Function(T) onUpdate;
  Function(Exception) onError;
  bool? renew;

  RequestCache(this.maxCacheTime, this.onUpdate, {this.onError = ignore, this.renew = false});

  static void ignore(Exception e) {}

  void start(String file, String document) async {
    Map<String, dynamic>? tableData = await Localstore.instance.collection(file).doc(document).get();
    if(tableData != null) {
      onUpdate(onLocalData(tableData['json']));
    }

    if(DateTime.now().millisecondsSinceEpoch - (maxCacheTime * 1000) < (tableData?['lastupdate'] ?? 0)) {
      if(renew == null || !renew!) return;
    }

    T? newValue;
    try {
      newValue = await onLoad();
      onUpdate(newValue as T);

      Localstore.instance.collection(file).doc(document).set({
        "json": jsonEncode(newValue),
        "lastupdate": DateTime.now().millisecondsSinceEpoch
      });
    } catch(e) {
      log("Error while fetching/ parsing. Raw server response: ${newValue?.rawResponse.body ?? "no response"}");
      onError(Exception(e.toString()));
    }
  }

  T onLocalData(String json);
  Future<T> onLoad();

}