import 'package:flutter/material.dart';
import 'package:marianum_mobile/view/pages/more/abiturCalculator/models/abiturCalculatorModel.dart';
import 'package:marianum_mobile/view/pages/more/abiturCalculator/models/subject.dart';
import 'package:marianum_mobile/view/pages/more/abiturCalculator/steps/resultStep.dart';
import 'package:marianum_mobile/view/pages/more/abiturCalculator/steps/selectGkStep.dart';
import 'package:marianum_mobile/view/pages/more/abiturCalculator/steps/selectLkStep.dart';
import 'package:marianum_mobile/view/pages/more/abiturCalculator/steps/selectPfStep.dart';
import 'package:marianum_mobile/view/pages/more/abiturCalculator/steps/welcomeStep.dart';
import 'package:provider/provider.dart';

class AbiturCalculatorView extends StatelessWidget {
  const AbiturCalculatorView({super.key});

  static RichText listSubjects(List<Subject> subjects) {
    return RichText(
      textAlign: TextAlign.center,
      text: TextSpan(
        children: subjects.map((subject) {
          return List<InlineSpan>.from([
            WidgetSpan(child: Icon(subject.icon, size: 15)),
            const WidgetSpan(child: SizedBox(width: 3)),
            TextSpan(text: subject.displayName),
            if(subjects.last != subject) const WidgetSpan(child: SizedBox(width: 10)),
          ]);
        }).expand((e) => e).toList(),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Consumer<AbiturCalculatorModel>(builder: (context, state, child) {
      return Scaffold(
          appBar: AppBar(
            title: const Text("Abitur Notenrechner"),
          ),
          body: Stepper(
            type: StepperType.vertical,
            steps: [
              const WelcomeStep(),
              SelectLkStep(),
              SelectGkStep(),
              SelectPfStep(),
              const ResultStep(),
            ],
            currentStep: state.getStep,
            onStepTapped: (step) => state.setStep = step,
            stepIconBuilder: (stepIndex, stepState) => _stepIconBuilder(context, stepIndex, stepState),
            controlsBuilder: _controlsBuilder,
          )
      );
    });
  }

  Widget _stepIconBuilder(BuildContext context, int stepIndex, StepState stepState) {
    return Consumer<AbiturCalculatorModel>(builder: (context, state, child) {
      return Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(20),
          color: state.getStep == stepIndex
              ? Theme.of(context).primaryColor
              : Theme.of(context).colorScheme.surface,
        ),
        child: Center(
          child: Text((++stepIndex).toString()),
        ),
      );
    });
  }

  Widget _controlsBuilder(BuildContext context, details) {
    return const Row(children: [SizedBox.shrink()]);
  }
}